package org.fuseri.modulelanguageschool.user;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityNotFoundException;
import jakarta.servlet.http.HttpServletRequest;
import lombok.Getter;
import org.fuseri.modulelanguageschool.common.DomainService;
import org.fuseri.modulelanguageschool.common.UserWithEmailAlreadyExists;
import org.fuseri.modulelanguageschool.course.Course;
import org.fuseri.modulelanguageschool.course.Language;
import org.fuseri.modulelanguageschool.course.ProficiencyLevel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.NativeWebRequest;

import java.util.Base64;
import java.util.List;
import java.util.Optional;

@Service
public class UserService extends DomainService<User> {

    @Getter
    private final UserRepository repository;

    @Autowired
    public UserService(UserRepository repository) {
        this.repository = repository;
    }

    @Override
    public User create(User entity) {
        if (repository.existsByEmail(entity.getEmail())) {
            throw new UserWithEmailAlreadyExists("User with " + entity.getEmail() + " already exists.");
        }
        return super.create(entity);
    }

    @Transactional(readOnly = true)
    public User find(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new EntityNotFoundException("User " + id + "' not found."));
    }

    public Optional<User> findUserByEmail(String email) {
        return repository.findByEmail(email);
    }

    public List<Course> getEnrolled(Long id) {
        return repository.getEnrolled(id);
    }

    public List<Course> getFinished(Long id) {
        return repository.getFinished(id);
    }

    public User addLanguageProficiency(Long id, Language language, ProficiencyLevel proficiency) {
        Optional<User> optionalUser = repository.findById(id);
        if (optionalUser.isPresent()) {
            User user = optionalUser.get();
            user.addLanguageProficiency(language, proficiency);
            return repository.save(user);
        } else {
            throw new EntityNotFoundException("User '" + id + "' not found.");
        }
    }

    public static String getEmailFromRequest(NativeWebRequest request) throws JsonProcessingException {
        // retrieve the HttpServletRequest object
        HttpServletRequest servletRequest = (HttpServletRequest) request.getNativeRequest();

        // extract the authorization header
        String authorizationHeader = servletRequest.getHeader("Authorization");

        // extract the bearer token
        String accessToken = authorizationHeader.substring(7);

        // decode the JWT token payload
        String[] jwtParts = accessToken.split("\\.");
        String jwtPayload = new String(Base64.getDecoder().decode(jwtParts[1]));

        // parse the JSON payload to retrieve the email
        ObjectMapper objectMapper = new ObjectMapper();
        JsonNode jsonNode = objectMapper.readTree(jwtPayload);
        return jsonNode.get("sub").asText();
    }
}
